@@ -3,13 +3,24 @@ import traceback
 from .tree_interpreter import TreeInterpreter
 import pandas as pd
 import numpy as np
+from tqdm import tqdm 
 
-from lime.lime_tabular import LimeTabularExplainer
+from joblib import delayed, Parallel 
+from joblib import wrap_non_picklable_objects
+from joblib.externals.loky import set_loky_pickler
+
+# This should allow for more shared memory for the LIME calcuations. 
+import os
+os.environ['JOBLIB_TEMP_FOLDER'] = '/tmp'
+
+###from lime.lime_tabular import LimeTabularExplainer
+from .lime_fast import FastLimeTabularExplainer
 
 from ..common.attributes import Attributes
 from ..common.importance_utils import retrieve_important_vars
 from ..common.utils import to_dataframe, determine_feature_dtype
 from ..common.metrics import brier_skill_score
+from ..common.multiprocessing_utils import tqdm_joblib, delayed, Parallel 
 
 
 from ..common.contrib_utils import (
@@ -26,6 +37,18 @@ list_of_acceptable_tree_estimators = [
     "ExtraTreesRegressor",
 ]
 
+def _ds_to_df(ds, method, estimator_name, feature_names):
+    """Convert the Dataset to DataFrame"""
+    df = pd.DataFrame(ds[f'{method}_values__{estimator_name}'].values, 
+                                           columns = feature_names)
+                   
+    df['Bias'] = ds[f'{method}_bias__{estimator_name}'].values
+              
+    X = pd.DataFrame(ds.X.values, columns=feature_names)
+    
+    return df, X 
+                    
+
 
 class LocalExplainer(Attributes):
 
@@ -98,13 +121,15 @@ class LocalExplainer(Attributes):
             self.feature_names = list(X.columns)
             self.estimator_output = estimator_output
 
-    def _get_local_prediction(
+    def _average_attributions(
         self,
-        method="shap",
+        method,
+        data=None, 
         performance_based=True,
         n_samples=100,
-        shap_kwargs=None,
+        shap_kws=None,
         lime_kws=None, 
+        n_jobs=1
     ):
         """
         Explain individual predictions using SHAP (SHapley Additive exPlanations;
@@ -113,14 +138,9 @@ class LocalExplainer(Attributes):
 
         Parameters
         ------------
-        method : 'treeinterpreter', 'shap', or 'lime'
-            Contributions can be computed using treeinterpreter for tree-based estimators
-            or using SHAP for both tree- and non-tree-based estimators
-        background_dataset : array (n_samples, n_features)
-            A representative (often a K-means or random sample) subset of the
-            data used to train the ML estimator. Used for the background dataset
-            to compute the expected values for the SHAP calculations.
-            Only required for non-tree based estimators.
+        data: xarray.Dataset 
+           Results of :func:`~ExplainToolkit.local_attributions`
+        
         performance_based : boolean (default=False)
             If True, will average feature contributions over the best and worst
             performing of the given X. The number of X to average over
@@ -128,7 +148,7 @@ class LocalExplainer(Attributes):
         n_samples : interger (default=100)
             Number of samples to compute average over if performance_based = True
 
-        shap_kwargs : dict
+        shap_kws : dict
             Arguments passed to the shap.Explainer object. See
             https://shap.readthedocs.io/en/latest/generated/shap.Explainer.html#shap.Explainer
             for details. The main two arguments supported in PyMint is the masker and
@@ -148,16 +168,6 @@ class LocalExplainer(Attributes):
         results_ds : pandas.DataFrame
 
         """
-        if method not in ["tree_interpreter", "shap", "lime"]:
-            raise ValueError(
-                """
-                             Declared method is not 'tree_interpreter','shap', or 'lime'. 
-                             Check for spelling mistake or syntax error!
-                             """
-            )
-        else:
-            self.method = method
-
         # will be returned; a list of pandas dataframes, one for each performance dict key
         contributions_dict = {
             estimator_name: {} for estimator_name in self.estimator_names
@@ -180,25 +190,39 @@ class LocalExplainer(Attributes):
                 )
 
                 for key, indices in performance_dict.items():
-                    cont_dict = self._get_feature_contributions(
+                    if data is not None:
+                        cont_dict, X = _ds_to_df(data, method, estimator_name, self.feature_names)
+                    else:
+                        X = self.X.iloc[indices, :]
+                        cont_dict = self._get_feature_contributions(
                         estimator=estimator,
-                        X=self.X.iloc[indices, :],
-                        shap_kwargs=shap_kwargs,
-                    )
+                        X=X,
+                        shap_kws=shap_kws,
+                        lime_kws=lime_kws,
+                        method=method, 
+                        n_jobs=n_jobs
+                        )
 
                     contributions_dict[estimator_name][key] = cont_dict
-                    feature_values_dict[estimator_name][key] = self.X.iloc[indices, :]
+                    feature_values_dict[estimator_name][key] = X
 
             else:
-                cont_dict = self._get_feature_contributions(
+                if data is not None:
+                    cont_dict, X = _ds_to_df(data, method, estimator_name, self.feature_names)
+                
+                else:
+                    cont_dict = self._get_feature_contributions(
                     estimator=estimator,
                     X=self.X,
-                    shap_kwargs=shap_kwargs,
-                    lime_kws=lime_kws, 
-                )
+                    shap_kws=shap_kws,
+                    lime_kws=lime_kws,
+                    method=method, 
+                    n_jobs=n_jobs
+                    )
+                    X = self.X
 
                 contributions_dict[estimator_name]["non_performance"] = cont_dict
-                feature_values_dict[estimator_name]["non_performance"] = self.X
+                feature_values_dict[estimator_name]["non_performance"] = X
 
             # average out the contributions and sort based on contribution
             avg_contrib_dict, avg_feature_val_dict = avg_and_sort_contributions(
@@ -218,23 +242,23 @@ class LocalExplainer(Attributes):
         self,
         estimator,
         X,
-        shap_kwargs=None,
+        shap_kws=None,
     ):
         """
         FOR INTERNAL PURPOSES ONLY.
 
         """
-        if shap_kwargs is None:
-            shap_kwargs = {}
+        if shap_kws is None:
+            shap_kws = {}
         
-        masker = shap_kwargs.get("masker", None)
-        algorithm = shap_kwargs.get("algorithm", "auto")
+        masker = shap_kws.get("masker", None)
+        algorithm = shap_kws.get("algorithm", "auto")
 
         if masker is None:
             raise ValueError(
-                """masker in shap_kwargs is None. 
+                """masker in shap_kws is None. 
                              This will cause issues with SHAP. We recommend starting with
-                             shap_kwargs = {'masker' = shap.maskers.Partition(X, max_samples=100, clustering="correlation")}
+                             shap_kws = {'masker' = shap.maskers.Partition(X, max_samples=100, clustering="correlation")}
                              where X is the original dataset and not the examples SHAP is being computed for. 
                              """
             )
@@ -272,7 +296,8 @@ class LocalExplainer(Attributes):
                             """
             )
 
-        ti = TreeInterpreter(estimator, X)
+        # TODO: generalize for the joint_contributions=True.
+        ti = TreeInterpreter(estimator, X, n_jobs=self._n_jobs)
 
         prediction, bias, contributions = ti.predict()
 
@@ -284,11 +309,31 @@ class LocalExplainer(Attributes):
 
         return contributions, bias
 
+    #@delayed
+    #@wrap_non_picklable_objects
+    def _explain_lime(self, explainer, predict_fn, X, label):
+        """Get explanation from LIME"""
+        
+        num_features = len(X)
+        contrib, bias = explainer.explain_instance(X, predict_fn, label=label, 
+                                                 num_features=num_features, num_samples=2500)
+        
+        #if isinstance(explainer, LimeTabularExplainer):
+        #    sorted_exp = sorted(explanation.local_exp[1], key=lambda x: x[0])
+        #    contrib = np.array([[val[1] for val in sorted_exp]])[0,:]
+        #    bias = explanation.intercept[1]
+        #else:
+        #contrib, bias = explanation
+
+        return contrib, bias 
+    
     
     def _get_lime_values(self, estimator, X, lime_kws):
         """
         Compute the Local Interpretable Model-Agnostic Explanations
         """
+        lime_kws['fast_lime'] = True
+        
         if lime_kws is None:
             raise KeyError('lime_kws is None, but lime_kws must contain training_data!')
         
@@ -296,16 +341,14 @@ class LocalExplainer(Attributes):
         if isinstance(lime_kws['training_data'], pd.DataFrame):
             lime_kws['training_data'] = lime_kws['training_data'].values
         
-        lime_kws['feature_names'] = X.columns
+        lime_kws['feature_names'] = list(X.columns)
         
         # Determine categorical features
         if lime_kws.get('categorical_names', None) is None and lime_kws.get('categorical_features', None) is None:
             features, cat_features = determine_feature_dtype(X, X.columns)
             lime_kws['categorical_names'] = cat_features
         
-        
         # Determine whether its a classification or regression task.
-        mode = 'regression' 
         if lime_kws.get('mode', None) is None:
             mode = 'classification' if hasattr(estimator, 'predict_proba') else 'regression'
             lime_kws['mode'] = mode
@@ -314,26 +357,47 @@ class LocalExplainer(Attributes):
         if lime_kws.get('random_state', None) is None:
             lime_kws['random_state'] = 123 
         
+        if lime_kws['fast_lime']:
+            del lime_kws['fast_lime']
+            lime_kws['feature_names'] = self.feature_names
+            explainer = FastLimeTabularExplainer(**lime_kws)
+        else:
+            del lime_kws['fast_lime']
+            explainer = LimeTabularExplainer(**lime_kws)
         
-        explainer = LimeTabularExplainer(**lime_kws)
-        
-        if mode == 'classification' and hasattr(estimator, 'predict_proba'):
+        if lime_kws['mode'] == 'classification' and hasattr(estimator, 'predict_proba'):
             predict_fn = estimator.predict_proba 
+            label = 1 
         else:
             predict_fn = estimator.predict 
+            label = 0 
         
-        # Using X.values[0] rather than X.values since it requires 
-        # a (n_features) rather than (1,n_features).
-        explanation = explainer.explain_instance(X.values[0], predict_fn, num_features=X.shape[1],)
-
-        sorted_exp = sorted(explanation.local_exp[1], key=lambda x: x[0])
-        contributions = np.array([[val[1] for val in sorted_exp]])
-        bias = explanation.intercept[1] 
+        n_examples = X.shape[0]
+        contributions = np.zeros(X.shape)
+        bias = np.zeros((n_examples))
+        
+        X_values = X.values 
     
+        # With the FAST-Lime code, parallelization is unneccsary. 
+        if self._n_jobs == -1:
+            parallel = Parallel(n_jobs=self._n_jobs, backend='loky')
+            with tqdm_joblib(tqdm(desc="LIME", total=n_examples)) as progress_bar:
+                results = parallel(delayed(self._explain_lime)(explainer, predict_fn, X_values[i,:], label) 
+                       for i in range(n_examples))
+       
+            for j, (contrib, b) in enumerate(results):
+                contributions[j,:] = contrib 
+                bias[j] = b
+        else:
+            for i in tqdm(range(n_examples), desc='LIME'):
+                contrib, b = self._explain_lime(explainer, predict_fn, X.values[i,:], label)
+                contributions[i,:] = contrib
+                bias[i] = b
+        
         return contributions, bias
         
-    
-    def _get_feature_contributions(self, estimator, X, shap_kwargs=None, lime_kws=None):
+    def _get_feature_contributions(self, estimator, X, n_jobs=1, shap_kws=None, 
+                                   lime_kws=None,  method=None, estimator_output=None):
         """
         FOR INTERNAL PURPOSES ONLY.
 
@@ -345,8 +409,16 @@ class LocalExplainer(Attributes):
         X : pandas.DataFrame of shape (n_samples, n_features)
 
         """
+        self._n_jobs=n_jobs
+        
+        if method is not None:
+            self.method = method
+        
+        if estimator_output is not None:
+            self.estimator_output = estimator_output
+        
         if self.method == "shap":
-            contributions, bias = self._get_shap_values(estimator, X, shap_kwargs)
+            contributions, bias = self._get_shap_values(estimator, X, shap_kws)
         elif self.method == "tree_interpreter":
             contributions, bias = self._get_ti_values(estimator, X)
         elif self.method == 'lime':
@@ -355,10 +427,6 @@ class LocalExplainer(Attributes):
         n_samples = len(X)
         columns = self.feature_names + ["Bias"]
 
-        if self.estimator_output == "probability":
-            contributions *= 100.0
-            bias *= 100.
-
         # A single example.
         if isinstance(bias, float):
             bias = [bias]
